from im2dhisteq import im2dhisteq
import numpy as np
import cv2


def test_im2dhisteq_with_param():
    image_name = "assets/Plane.jpg"
    image = cv2.imread(image_name)
    # convert rgb image to gray
    gray_image = cv2.cvtColor(image, cv2.COLOR_BGR2GRAY)

    gray_image_2DHisteq = im2dhisteq(gray_image, w_neighboring=6)

    # np.save(f'{image_name}-2D-Histogram-Equalized', gray_image_2DHisteq)
    gray_image_2DHisteq_cmpr = np.load(f"{image_name}-2D-Histogram-Equalized.npy")
    assert np.all(gray_image_2DHisteq == gray_image_2DHisteq_cmpr)



from im2dhist import im2dhist as im2dhist
from im2dhist import imhist
import numba
import numpy as np


@numba.njit()
def transformer(image, w_neighboring=6):
    [h, w] = image.shape
    image_hist = imhist(image)
    H_in = im2dhist(image, w_neighboring=w_neighboring)
    CDFx = np.cumsum(np.sum(H_in, axis=0))  # Kx1

    # normalizes CDFx
    CDFxn = CDFx / CDFx[-1]

    PDFxn = np.zeros_like(CDFxn)
    PDFxn[0] = CDFxn[0]
    PDFxn[1:] = np.diff(CDFxn)

    X_transform = np.zeros((256))
    X_transform[np.where(image_hist > 0)] = PDFxn.copy()
    CDFxn_transform = np.cumsum(X_transform)
    return CDFxn_transform, H_in


@numba.njit()
def im2dhisteq(image, w_neighboring=6):
    [h, w] = image.shape
    V = image.copy()
    CDFxn_transform, _ = transformer(image, w_neighboring=w_neighboring)
    bins = np.arange(256)
    # use linear interpolation of cdf to find new pixel values
    image_equalized = np.floor(
        np.interp(V.flatten(), bins, 255 * CDFxn_transform).reshape(h, w)
    ).astype(np.uint8)

    return image_equalized


@numba.njit()
def vid2dhisteq(image, w_neighboring=6, Wout_list=np.zeros((10))):
    h, w = image.shape

    V = image
    CDFxn_transform, H_in = transformer(image, w_neighboring=w_neighboring)

    Win = H_in.shape[0]
    Gmax = 1.5  # 1.5 .. 2
    Wout = min(255, Gmax * Win)
    if np.where(Wout_list > 0)[0].size == Wout_list.size:
        Wout = (np.sum(Wout_list) + Wout) / (1 + Wout_list.size)

    F = V.ravel().astype(np.uint16)
    Ftilde = Wout * CDFxn_transform[F]

    Madj = np.mean(V) - np.mean(Ftilde)
    Ftilde = Ftilde + Madj

    Ftilde = np.where(Ftilde >= 0, Ftilde, np.zeros_like(Ftilde))
    Ftilde = np.where(Ftilde <= 255, Ftilde, 255 * np.ones_like(Ftilde))
    Ftilde = Ftilde.astype(np.uint8)

    image_equalized = Ftilde.reshape(h, w)

    return image_equalized, Wout



from setuptools import setup, find_packages
import codecs
import os

parent_dir = os.path.abspath(os.path.dirname(__file__))

with codecs.open(os.path.join(parent_dir, "README.md"), encoding="utf-8") as readme:
    readme_description = "\n" + readme.read()


setup(
    name="im2dhisteq",
    version="1.0.3",
    author="mamdasn s",
    author_email="<mamdassn@gmail.com>",
    url="https://github.com/Mamdasn/im2dhisteq",
    description="This module attempts to enhance contrast of a given image by equalizing its two dimensional histogram.",
    long_description=readme_description,
    long_description_content_type="text/markdown",
    include_package_data=True,
    package_dir={"": "src"},
    py_modules=["im2dhisteq"],
    install_requires=[
        "numpy",
        "numba",
        "im2dhist",
    ],
    keywords=[
        "python",
        "histogram",
        "image-processing",
        "contrast-enhancement",
        "histogram-equalization",
        "image-contrast-enhancement",
        "imhist",
        "2dhist",
        "hist2d",
        "im2dhisteq",
        "two-dimensional-histogram",
    ],
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Programming Language :: Python :: 3.8",
        "Operating System :: OS Independent",
    ],
    scripts=["bin/im2dhisteq", "bin/vid2dhisteq"],
)
